#!/usr/bin/perl -w
#
# Plot - plot time-based data on screen and to file with interactive controls
# Copyright (C) 2006  Jonas Berlin <xkr47@outerspace.dyndns.org>
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
####
# xcpp - extended C preprocessor that supports multiline macros which
# also allow #defines to be used inside

use strict;

my %templates;

my %files;

my $err = 0;

sub read_file {
    my ($path, $fd) = @_;

    my $deppath = $path.".d";
    my $depfh;
    open $depfh, '>', $deppath;
    print $depfh $path,": \\\n";

    $files{$path} = 1;

    print "#define _GNU_SOURCE 1\n";
    print '#line 1 "'.$path."\"\n";

    my $line = 0;
    while(<$fd>) {
	++$line;
#     unless(m!^(.*)<<(\S+)\s*\{!) {
# 	print;
# 	next;
#     }

#     my ($pre, $key) = ($1, $2);
#     print $pre."\\\n";

#     while(<$fd>) {
# 	chomp;
# 	if(m!^\}\s*$key\s*;\s*$!) {
# 	    print "\n";
# 	    last;
# 	}
# 	print $_."\\\n";
#     }

#######################################

	if(/^\#deftemplate\s+(\S+)\s*\(\s*(\S+(?:\s*,\s*\S+)*)\s*\)\s*$/) {
	    my ($name, $args) = ($1,$2);
	    if(defined($templates{$name})) {
		my $t = $templates{$name};
		print STDERR $path.':'.$line.": error: template '$name' already defined\n";
		print STDERR $t->[3].':'.$t->[2].": .. here\n";
		$err = 1;
	    }
	    my @args = split(/\s*,\s*/, $args);
	    my @lines;
	    my $firstline = $line + 1;
	    while(<$fd>) {
		++$line;
		last if(/^\#endtemplate\b/);
		push @lines, $_;
	    }
	    $templates{$name} = [ \@args, \@lines, $firstline, $path ] unless(defined($templates{$name}));
	    next;
	}

	if(/^\#template\s+(\S+)\s*\(\s*(\S*(?:\s*,\s*\S*)*)\s*\)\s*$/) {
	    my ($name, $args) = ($1,$2);
	    my @args = split(/\s*,\s*/, $args);

	    my $tref = $templates{$name};
	    unless(defined($tref)) {
		print STDERR $path.':'.$line.": no such template $name\n";
		print "\n";
		next;
	    }
	    my ($targs, $tlines, $firstline, $tpath) = @$tref;

	    unless($#{$targs} == $#args) {
		print STDERR $path.':'.$line.": wrong number of arguments (expecting ".($#{$targs}+1).", got ".($#args+1).")\n";
		exit(1);
	    }

	    print '#line '.$firstline.' "'.$tpath."\"\n";
	    foreach (@$tlines) {
		my $line = $_;
		for(my $i=0; $i<=$#{$targs}; ++$i) {
#		$line =~ s/(?<![a-zA-Z0-9])$targs->[$i](?![a-zA-Z0-9])/$args[$i]/g;
		    $line =~ s/$targs->[$i]/$args[$i]/g;
		}
		print $line;
	    }
	    print '#line '.($line+1).' "'.$path."\"\n";
	    next;
	}

	if(/^\#include\s+\"([^\"]+\.xh)\"/) {
	    my $newfilename = $1;
	    my $newpath = $path;
	    $newpath =~ s![^/]+$!!;
	    $newpath .= $newfilename;
	    
	    print $depfh "\t",$newpath," \\\n";

	    if(defined($files{$newpath})) {
		#print STDERR $path.':'.$line.": Already did $newpath\n";
		print "\n";
		next;
	    }

	    my $newfd;
	    unless(open $newfd, '<', $newpath) {
		print STDERR $path.':'.$line.": error: file '$newpath' not found\n";
		$err = 1;
		print "\n";
		next;
	    }

	    read_file($newpath, $newfd);

	    close($newfd);
#	print '#line 1 "'.$path."\"\n";
#	    my $pathdef = "__".$newpath."_";
#	    $pathdef =~ s/[^a-zA-Z_]/_/g;
	    #print "#ifndef $pathdef\n#define $pathdef\n";
#	    while(<F>) {
#		print;
#	    }
	    #print "#endif\n";
	    print '#line '.($line+1).' "'.$path."\"\n";
	    next;
	}

	print;
    }

    print $depfh "\n";
    close $depfh;
}

read_file($ARGV[0], *STDIN);

exit($err);
